#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
#  veritar: In-place verification of the MD5 sums of files within a tar archive.
#
#  Project: https://www.codetrax.org/svn/veritar
#
#  Copyright 2007 George Notaras <gnot [at] g-loaded.eu>, CodeTRAX.org
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

__version__ = "0.1.0"
__author__ = "George Notaras <gnot -at- g-loaded.eu>"
__credits__ = ""

import os
import sys
import tarfile
#import hashlib
import md5	# for compatibility with older Python versions

def get_md5sum(f):
	READ_BLOCK = 8192
	#m = hashlib.md5()
	m = md5.new()
	data = f.read(READ_BLOCK)
	while data:
		m.update(data)
		data = f.read(READ_BLOCK)
	return m.hexdigest()

def get_valid_checksums(path):
	csums = {}
	f_sums = open(path)
	for line in f_sums:
		if line.strip():
			csum, name = line.split("  ")
			csums[name.strip()] = csum.strip()
	f_sums.close()
	return csums

def check_tar_members(tarfile_path, md5file_path):
	csums = get_valid_checksums(md5file_path)
	f_tar = tarfile.open(tarfile_path, "r|")
	f_tar.ignore_zeros = True
	f_tar.debug = 1
	f_tar.errorlevel = 2
	sys.stdout.write("%s v%s\n" % \
				(os.path.basename(sys.argv[0]), __version__))
	sys.stdout.write("\nTAR members checksum verification\n")
	sys.stdout.write("---------------------------------\n")
	sys.stdout.flush()
	member = f_tar.next()
	while member:
		f = f_tar.extractfile(member)
		if f:
			checksum = get_md5sum(f)
			f.close()
			if checksum == csums[member.name]:
				sys.stdout.write("OK %s\n" % \
							member.name.rjust(16))
			else:
				sys.stdout.write("CORRUPT %s\n" % \
							member.name.rjust(16))
			sys.stdout.flush()
			del csums[member.name]
		else:
			sys.stderr.write("Skipping non-file: %s\n" % \
								member.name)
			sys.stderr.flush()
		member = f_tar.next()
	f_tar.close()
	if csums:
		sys.stdout.write("\n\nPossibly corrupted files\n")
		sys.stdout.write("------------------------\n")
		for remnant in csums.keys():
			sys.stdout.write("CORRUPT %s\n" % remnant.rjust(16))
	sys.stdout.write("\n")
	sys.stdout.flush()

def usage():
	sys.stdout.write("""%(program)s v%(version)s

Usage:

    %(program)s tarfile md5sumfile

    md5sumfile format:  'md5sum  path'
    
""" % {"program":os.path.basename(sys.argv[0]), "version":__version__})
	sys.stdout.flush()
	sys.exit(1)

def main():
	if len(sys.argv) != 3:
		usage()
	elif sys.argv.count("-h") or sys.argv.count("--help"):
		usage()
	else:
		check_tar_members(sys.argv[1], sys.argv[2])

if __name__ == '__main__':
	main()

